from base_st_classes import *

class ReadlinkSuite(NFSSuite):
    """Test operation 27: READLINK

    Equivalence partitioning:

    Input Condition: current filehandle
        Valid equivalence classes:
            symlink(10)
        Invalid equivalence classes:
            not symlink or directory(11)
            directory(12)
            no filehandle(13)
    """
    def _readlink(self,pathcomps=None,error=[NFS4_OK]):
        if not pathcomps:
            pathcomps = self.regfile
        ops = [self.ncl.putrootfh_op()] + self.ncl.lookup_path(pathcomps)
        ops.append(self.ncl.readlink_op())
        res = self.ncl.do_ops(ops)
        self.assert_status(res,error)
        return res

    #
    # Testcases covering valid equivalence classes.
    #
    def testReadlink(self):
        """READLINK on link

        Covered valid equivalence classes: 10
        """
        res = self._readlink(self.linkfile)
        linkdata = res.resarray[-1].arm.arm.link
        self.failIf(linkdata != "fd0",
                    "link data was %s, should be fd0" % linkdata)
    #
    # Testcases covering invalid equivalence classes.
    #
    def testFhNotSymlink(self):
        """READLINK on non-symlink objects should return NFS4ERR_INVAL

        Covered valid equivalence classes: 11, 12
        """
        for pathcomps in [self.regfile,
                          self.blockfile,
                          self.charfile,
                          self.socketfile,
                          self.fifofile,
                          self.dirfile]:
            self._readlink(pathcomps,error=[NFS4ERR_INVAL])

    def testNoFh(self):
        """READLINK without (cfh) should return NFS4ERR_NOFILEHANDLE

        Covered invalid equivalence classes: 13
        """
        readlinkop = self.ncl.readlink_op()
        res = self.ncl.do_ops([readlinkop])
        self.assert_status(res, [NFS4ERR_NOFILEHANDLE])
